@@ -9,12 +9,15 @@
 import numpy as np
 
 from grid2op.dtypes import dt_int, dt_bool, dt_float
-from grid2op.Exceptions import *
 from grid2op.Space import GridObjects
 
 
 # TODO see if it can be done in c++ easily
 class ValueStore:
+    """
+    .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
+
+    """
     def __init__(self, size, dtype):
         ## TODO at the init it's mandatory to have everything at "1" here
         # if topo is not "fully connected" it will not work
@@ -55,6 +58,10 @@ class ValueStore:
         self.last_index = 0
 
     def change_status(self, switch, lineor_id, lineex_id, old_vect):
+        if not np.any(switch):
+            # nothing is modified so i stop here
+            return
+
         # changed
         changed_ = switch
 
@@ -155,12 +162,17 @@ class ValueStore:
         outside of this usecase"""
         self.changed[:] = self.changed[new_order]
         self.values[:] = self.values[new_order]
-        # self.changed[new_order] = self.changed[:]
-        # self.values[new_order] = self.values[:]
+
+    def copy_from_index(self, ref, index):
+        self.reset()
+        self.changed[:] = ref.changed[index]
+        self.values[:] = ref.values[index]
 
 
 class _BackendAction(GridObjects):
     """
+    .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
+
     Internal class, use at your own risk.
 
     This class "digest" the players / environment / opponent / voltage controlers "action",
@@ -194,8 +206,16 @@ class _BackendAction(GridObjects):
         self._status_or = np.ones(self.n_line, dtype=dt_int)
         self._status_ex = np.ones(self.n_line, dtype=dt_int)
 
+        self._loads_bus = None
+        self._gens_bus = None
+        self._lines_or_bus = None
+        self._lines_ex_bus = None
+
     def reorder(self, no_load, no_gen, no_topo, no_shunt):
-        """reorder the element modified, this is use when converting backends only and should not be use
+        """
+        .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
+
+        reorder the element modified, this is use when converting backends only and should not be use
         outside of this usecase
 
         no_* stands for "new order"
@@ -255,6 +275,8 @@ class _BackendAction(GridObjects):
 
     def __iadd__(self, other):
         """
+        .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
+
         other: a grid2op action standard
 
         Parameters
@@ -275,20 +297,22 @@ class _BackendAction(GridObjects):
 
         # I deal with injections
         # Ia set the injection
-        if "load_p" in dict_injection:
-            tmp = dict_injection["load_p"]
-            self.load_p.set_val(tmp)
-        if "load_q" in dict_injection:
-            tmp = dict_injection["load_q"]
-            self.load_q.set_val(tmp)
-        if "prod_p" in dict_injection:
-            tmp = dict_injection["prod_p"]
-            self.prod_p.set_val(tmp)
-        if "prod_v" in dict_injection:
-            tmp = dict_injection["prod_v"]
-            self.prod_v.set_val(tmp)
+        if other._modif_inj:
+            if "load_p" in dict_injection:
+                tmp = dict_injection["load_p"]
+                self.load_p.set_val(tmp)
+            if "load_q" in dict_injection:
+                tmp = dict_injection["load_q"]
+                self.load_q.set_val(tmp)
+            if "prod_p" in dict_injection:
+                tmp = dict_injection["prod_p"]
+                self.prod_p.set_val(tmp)
+            if "prod_v" in dict_injection:
+                tmp = dict_injection["prod_v"]
+                self.prod_v.set_val(tmp)
         # Ib change the injection aka redispatching
-        self.prod_p.change_val(redispatching)
+        if other._modif_redispatch:
+            self.prod_p.change_val(redispatching)
 
         # II shunts
         if self.shunts_data_available:
@@ -308,51 +332,51 @@ class _BackendAction(GridObjects):
         # III line status
         # this need to be done BEFORE the topology, as a connected powerline will be connected to their old bus.
         # regardless if the status is changed in the action or not.
-        self.current_topo.change_status(switch_status,
-                                        self.line_or_pos_topo_vect,
-                                        self.line_ex_pos_topo_vect,
-                                        self.last_topo_registered)
-        self.current_topo.set_status(set_status,
-                                     self.line_or_pos_topo_vect,
-                                     self.line_ex_pos_topo_vect,
-                                     self.last_topo_registered)
+        if other._modif_change_status:
+            self.current_topo.change_status(switch_status,
+                                            self.line_or_pos_topo_vect,
+                                            self.line_ex_pos_topo_vect,
+                                            self.last_topo_registered)
+        if other._modif_set_status:
+            self.current_topo.set_status(set_status,
+                                         self.line_or_pos_topo_vect,
+                                         self.line_ex_pos_topo_vect,
+                                         self.last_topo_registered)
 
-        topo_before = self.current_topo.values
-        self._status_or_before[:], \
-        self._status_ex_before[:] = self.current_topo.get_line_status(
-            self.line_or_pos_topo_vect,
-            self.line_ex_pos_topo_vect)
+        # if other._modif_change_status or other._modif_set_status:
+        self._status_or_before[:], self._status_ex_before[:] = self.current_topo.get_line_status(
+            self.line_or_pos_topo_vect, self.line_ex_pos_topo_vect)
 
         # IV topo
-        self.current_topo.change_val(switcth_topo_vect)
-        self.current_topo.set_val(set_topo_vect)
+        if other._modif_change_bus:
+            self.current_topo.change_val(switcth_topo_vect)
+        if other._modif_set_bus:
+            self.current_topo.set_val(set_topo_vect)
 
         # V Force disconnected status
         # of disconnected powerlines extremities
-        self._status_or[:], \
-        self._status_ex[:] = self.current_topo.get_line_status(self.line_or_pos_topo_vect,
-                                                               self.line_ex_pos_topo_vect)
+        self._status_or[:], self._status_ex[:] = self.current_topo.get_line_status(
+            self.line_or_pos_topo_vect, self.line_ex_pos_topo_vect)
 
         # At least one disconnected extremity
-        disco_or = (self._status_or_before == -1) | (self._status_or == -1)
-        disco_ex = (self._status_ex_before == -1) | (self._status_ex == -1)
-        disco_now = disco_or | disco_ex  # a powerline is disconnected if at least one of its extremity is
-        #
-        # added
-        reco_or = (self._status_or_before == -1) & (self._status_or >= 1)
-        reco_ex = (self._status_or_before == -1) & (self._status_ex >= 1)
-        reco_now = reco_or | reco_ex
-        #
-        # Set nothing
-        set_now = np.zeros_like(self._status_or)
-        # # Force some disconnections
-        set_now[disco_now] = -1
-        set_now[reco_now] = 1
+        if other._modif_change_bus or other._modif_set_bus:
+            disco_or = (self._status_or_before == -1) | (self._status_or == -1)
+            disco_ex = (self._status_ex_before == -1) | (self._status_ex == -1)
+            disco_now = disco_or | disco_ex  # a powerline is disconnected if at least one of its extremity is
+            # added
+            reco_or = (self._status_or_before == -1) & (self._status_or >= 1)
+            reco_ex = (self._status_or_before == -1) & (self._status_ex >= 1)
+            reco_now = reco_or | reco_ex
+            # Set nothing
+            set_now = np.zeros_like(self._status_or)
+            # Force some disconnections
+            set_now[disco_now] = -1
+            set_now[reco_now] = 1
 
-        self.current_topo.set_status(set_now,
-                                     self.line_or_pos_topo_vect,
-                                     self.line_ex_pos_topo_vect,
-                                     self.last_topo_registered)
+            self.current_topo.set_status(set_now,
+                                         self.line_or_pos_topo_vect,
+                                         self.line_ex_pos_topo_vect,
+                                         self.last_topo_registered)
 
         return self
 
@@ -365,6 +389,30 @@ class _BackendAction(GridObjects):
         self._get_active_bus()
         return self.activated_bus, injections, topo, shunts
 
+    def get_loads_bus(self):
+        if self._loads_bus is None:
+            self._loads_bus = ValueStore(self.n_load, dtype=dt_int)
+        self._loads_bus.copy_from_index(self.current_topo, self.load_pos_topo_vect)
+        return self._loads_bus
+
+    def get_gens_bus(self):
+        if self._gens_bus is None:
+            self._gens_bus = ValueStore(self.n_gen, dtype=dt_int)
+        self._gens_bus.copy_from_index(self.current_topo, self.gen_pos_topo_vect)
+        return self._gens_bus
+
+    def get_lines_or_bus(self):
+        if self._lines_or_bus is None:
+            self._lines_or_bus = ValueStore(self.n_line, dtype=dt_int)
+        self._lines_or_bus.copy_from_index(self.current_topo, self.line_or_pos_topo_vect)
+        return self._lines_or_bus
+
+    def get_lines_ex_bus(self):
+        if self._lines_ex_bus is None:
+            self._lines_ex_bus = ValueStore(self.n_line, dtype=dt_int)
+        self._lines_ex_bus.copy_from_index(self.current_topo, self.line_ex_pos_topo_vect)
+        return self._lines_ex_bus
+
     def _get_active_bus(self):
         self.activated_bus[:] = False
         tmp = self.current_topo.values-1
@@ -372,14 +420,17 @@ class _BackendAction(GridObjects):
 
     def update_state(self, powerline_disconnected):
         """
+        .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
+
         Update the internal state. Should be called after the cascading failures
 
         """
-        arr_ = np.zeros(powerline_disconnected.shape, dtype=dt_int)
-        arr_[powerline_disconnected] = -1
-        self.current_topo.set_status(arr_,
-                                     self.line_or_pos_topo_vect,
-                                     self.line_ex_pos_topo_vect,
-                                     self.last_topo_registered)
+        if np.any(powerline_disconnected):
+            arr_ = np.zeros(powerline_disconnected.shape, dtype=dt_int)
+            arr_[powerline_disconnected] = -1
+            self.current_topo.set_status(arr_,
+                                         self.line_or_pos_topo_vect,
+                                         self.line_ex_pos_topo_vect,
+                                         self.last_topo_registered)
         self.last_topo_registered.update_connected(self.current_topo)
         self.current_topo.reset()
