@@ -6,14 +6,6 @@
 # SPDX-License-Identifier: MPL-2.0
 # This file is part of Grid2Op, Grid2Op a testbed platform to model sequential decision making in power systems.
 
-"""
-This module presents an example of an implementation of a `grid2op.Backend` when using the powerflow
-implementation "pandapower" available at `PandaPower <https://www.pandapower.org/>`_ for more details about
-this backend. This file is provided as an example of a proper :class:`grid2op.Backend.Backend` implementation.
-
-This backend currently does not work with 3 winding transformers and other exotic object.
-"""
-
 import os  # load the python os default module
 import sys  # laod the python sys default module
 import copy
@@ -41,6 +33,18 @@ except (ImportError, ModuleNotFoundError):
 
 class PandaPowerBackend(Backend):
     """
+    .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
+
+        If you want to code a backend to use grid2op with another powerflow, you can get inspired
+        from this class. Note However that implies knowning the behaviour
+        of PandaPower.
+
+    This module presents an example of an implementation of a `grid2op.Backend` when using the powerflow
+    implementation "pandapower" available at `PandaPower <https://www.pandapower.org/>`_ for more details about
+    this backend. This file is provided as an example of a proper :class:`grid2op.Backend.Backend` implementation.
+
+    This backend currently does not work with 3 winding transformers and other exotic object.
+
     As explained in the `grid2op.Backend` module, every module must inherit the `grid2op.Backend` class.
 
     This class have more attributes that are used internally for faster information retrieval.
@@ -83,6 +87,20 @@ class PandaPowerBackend(Backend):
     v_ex: :class:`numpy.array`, dtype:float
         The voltage magnitude at the extremity bus of the powerline
 
+    Examples
+    ---------
+    The only recommended way to use this class is by passing an instance of a Backend into the "make"
+    function of grid2op. Do not attempt to use a backend outside of this specific usage.
+
+    .. code-block:: python
+
+            import grid2op
+            from grid2op.Backend import PandaPowerBackend
+            backend = PandaPowerBackend()
+
+            env = grid2op.make(backend=backend)
+            # and use "env" as any open ai gym environment.
+
     """
     def __init__(self, detailed_infos_for_cascading_failures=False):
         Backend.__init__(self, detailed_infos_for_cascading_failures=detailed_infos_for_cascading_failures)
@@ -111,7 +129,7 @@ class PandaPowerBackend(Backend):
 
         self._pf_init = "flat"
         self._pf_init = "results"
-        self._nb_bus_before = 0
+        self._nb_bus_before = None  # number of active bus at the preceeding step
 
         self.thermal_limit_a = None
 
@@ -128,6 +146,19 @@ class PandaPowerBackend(Backend):
         self.cst_1 = dt_float(1.0)
         self._topo_vect = None
         self.slack_id = None
+        self.comp_time = 0.
+
+        # function to rstore some information
+        self.__nb_bus_before = None  # number of substation in the powergrid
+        self.__nb_powerline = None  # number of powerline (real powerline, not transformer)
+        self._init_bus_load = None
+        self._init_bus_gen = None
+        self._init_bus_lor = None
+        self._init_bus_lex = None
+        self._get_vector_inj = None
+        self._big_topo_to_obj = None
+        self._big_topo_to_backend = None
+        self.__pp_backend_initial_state = None  # initial state to facilitate the "reset"
 
         # Mapping some fun to apply bus updates
         self._type_to_bus_set = [
@@ -141,6 +172,8 @@ class PandaPowerBackend(Backend):
 
     def get_nb_active_bus(self):
         """
+        .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
+
         Compute the amount of buses "in service" eg with at least a powerline connected to it.
 
         Returns
@@ -168,17 +201,24 @@ class PandaPowerBackend(Backend):
 
     def reset(self, path=None, grid_filename=None):
         """
+        .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
+
         Reload the grid.
         For pandapower, it is a bit faster to store of a copy of itself at the end of load_grid
         and deep_copy it to itself instead of calling load_grid again
         """
         # Assign the content of itself as saved at the end of load_grid
         # This overide all the attributes with the attributes from the copy in __pp_backend_initial_state
-        self.__dict__.update(copy.deepcopy(self.__pp_backend_initial_state).__dict__)
+        # self.__dict__.update(copy.deepcopy(self.__pp_backend_initial_state).__dict__)
+        self._grid = copy.deepcopy(self.__pp_backend_initial_state._grid)
+        self._reset_all_nan()
         self._topo_vect[:] = self._get_topo_vect()
+        self.comp_time = 0.
 
     def load_grid(self, path=None, filename=None):
         """
+        .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
+
         Load the _grid, and initialize all the member of the class. Note that in order to perform topological
         modification of the substation of the underlying powergrid, some buses are added to the test case loaded. They
         are set as "out of service" unless a topological action acts on these specific substations.
@@ -186,7 +226,7 @@ class PandaPowerBackend(Backend):
         """
 
         if path is None and filename is None:
-            raise RuntimeError("You must provide at least one of path or file to laod a powergrid.")
+            raise RuntimeError("You must provide at least one of path or file to load a powergrid.")
         if path is None:
             full_path = filename
         elif filename is None:
@@ -414,6 +454,7 @@ class PandaPowerBackend(Backend):
             name_shunt.append("shunt_{bus}_{index_shunt}".format(**row, index_shunt=i))
             self.shunt_to_subid[i] = bus
         self.name_shunt = np.array(name_shunt)
+        self._sh_vnkv = self._grid.bus["vn_kv"][self.shunt_to_subid].values.astype(dt_float)
         self.shunts_data_available = True
 
         # store the topoid -> objid
@@ -456,6 +497,8 @@ class PandaPowerBackend(Backend):
 
     def _convert_id_topo(self, id_big_topo):
         """
+        .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
+
         convert an id of the big topo vector into:
 
         - the id of the object in its "only object" (eg if id_big_topo represents load 2, then it will be 2)
@@ -466,6 +509,8 @@ class PandaPowerBackend(Backend):
 
     def apply_action(self, backendAction=None):
         """
+        .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
+
         Specific implementation of the method to apply an action modifying a powergrid in the pandapower format.
         """
         if backendAction is None:
@@ -603,6 +648,8 @@ class PandaPowerBackend(Backend):
 
     def runpf(self, is_dc=False):
         """
+        .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
+
         Run a power flow on the underlying _grid. This implements an optimization of the powerflow
         computation: if the number of
         buses has not changed between two calls, the previous results are re used. This speeds up the computation
@@ -635,17 +682,35 @@ class PandaPowerBackend(Backend):
                 else:
                     pp.runpp(self._grid, check_connectivity=False, init=self._pf_init, numba=numba_)
 
+                # stores the computation time
+                if "_ppc" in self._grid:
+                    if "et" in self._grid["_ppc"]:
+                        self.comp_time += self._grid["_ppc"]["et"]
                 if self._grid.res_gen.isnull().values.any():
                     # TODO see if there is a better way here -> do not handle this here, but rather in Backend._next_grid_state
                     # sometimes pandapower does not detect divergence and put Nan.
                     raise pp.powerflow.LoadflowNotConverged("Isolated gen")
 
+                self.prod_p[:], self.prod_q[:], self.prod_v[:] = self._gens_info()
                 self.load_p[:], self.load_q[:], self.load_v[:] = self._loads_info()
                 if not is_dc:
                     if not np.all(np.isfinite(self.load_v)):
                         # TODO see if there is a better way here
                         # some loads are disconnected: it's a game over case!
                         raise pp.powerflow.LoadflowNotConverged("Isolated load")
+                else:
+                    # fix voltages magnitude that are always "nan" for dc case
+                    # self._grid.res_bus["vm_pu"] is always nan when computed in DC
+                    self.load_v[:] = self.load_pu_to_kv  # TODO
+                    # need to assign the correct value when a generator is present at the same bus
+                    # TODO optimize this ugly loop
+                    for l_id in range(self.n_load):
+                        if self.load_to_subid[l_id] in self.gen_to_subid:
+                            ind_gens = np.where(self.gen_to_subid == self.load_to_subid[l_id])[0]
+                            for g_id in ind_gens:
+                                if self._topo_vect[self.load_pos_topo_vect[l_id]] == self._topo_vect[self.load_pos_topo_vect[g_id]]:
+                                    self.load_v[l_id] = self.prod_v[g_id]
+                                    break
 
                 self.line_status[:] = self._get_line_status()
                 # I retrieve the data once for the flows, so has to not re read multiple dataFrame
@@ -656,10 +721,6 @@ class PandaPowerBackend(Backend):
                 self.a_or[~np.isfinite(self.a_or)] = 0.
                 self.v_or[~np.isfinite(self.v_or)] = 0.
 
-                # it seems that pandapower does not take into account disconencted powerline for their voltage
-                self.v_or[~self.line_status] = 0.
-                self.v_ex[~self.line_status] = 0.
-
                 self.p_ex[:] = self._aux_get_line_info("p_to_mw", "p_lv_mw")
                 self.q_ex[:] = self._aux_get_line_info("q_to_mvar", "q_lv_mvar")
                 self.v_ex[:] = self._aux_get_line_info("vm_to_pu", "vm_lv_pu")
@@ -667,11 +728,13 @@ class PandaPowerBackend(Backend):
                 self.a_ex[~np.isfinite(self.a_ex)] = 0.
                 self.v_ex[~np.isfinite(self.v_ex)] = 0.
 
+                # it seems that pandapower does not take into account disconencted powerline for their voltage
+                self.v_or[~self.line_status] = 0.
+                self.v_ex[~self.line_status] = 0.
+
                 self.v_or[:] *= self.lines_or_pu_to_kv
                 self.v_ex[:] *= self.lines_ex_pu_to_kv
 
-                self.prod_p[:], self.prod_q[:], self.prod_v[:] = self._gens_info()
-
                 self._nb_bus_before = None
                 self._grid._ppc["gen"][self._iref_slack, 1] = 0.
                 self._topo_vect[:] = self._get_topo_vect()
@@ -679,26 +742,30 @@ class PandaPowerBackend(Backend):
 
         except pp.powerflow.LoadflowNotConverged as exc_:
             # of the powerflow has not converged, results are Nan
-            self.p_or[:] = np.NaN
-            self.q_or[:] = np.NaN
-            self.v_or[:] = np.NaN
-            self.a_or[:] = np.NaN
-            self.p_ex[:] = np.NaN
-            self.q_ex[:] = np.NaN
-            self.v_ex[:] = np.NaN
-            self.a_ex[:] = np.NaN
-            self.prod_p[:] = np.NaN
-            self.prod_q[:] = np.NaN
-            self.prod_v[:] = np.NaN
-            self.load_p[:] = np.NaN
-            self.load_q[:] = np.NaN
-            self.load_v[:] = np.NaN
-
-            self._nb_bus_before = None
+            self._reset_all_nan()
             return False
 
+    def _reset_all_nan(self):
+        self.p_or[:] = np.NaN
+        self.q_or[:] = np.NaN
+        self.v_or[:] = np.NaN
+        self.a_or[:] = np.NaN
+        self.p_ex[:] = np.NaN
+        self.q_ex[:] = np.NaN
+        self.v_ex[:] = np.NaN
+        self.a_ex[:] = np.NaN
+        self.prod_p[:] = np.NaN
+        self.prod_q[:] = np.NaN
+        self.prod_v[:] = np.NaN
+        self.load_p[:] = np.NaN
+        self.load_q[:] = np.NaN
+        self.load_v[:] = np.NaN
+        self._nb_bus_before = None
+
     def copy(self):
         """
+        .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
+
         Performs a deep copy of the power :attr:`_grid`.
         As pandapower is pure python, the deep copy operator is perfectly suited for the task.
         """
@@ -707,6 +774,8 @@ class PandaPowerBackend(Backend):
 
     def close(self):
         """
+        .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
+
         Called when the :class:`grid2op;Environment` has terminated, this function only reset the grid to a state
         where it has not been loaded.
         """
@@ -715,6 +784,10 @@ class PandaPowerBackend(Backend):
 
     def save_file(self, full_path):
         """
+        .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
+
+        You might want to use it for debugging purpose only, and only if you develop yourself a backend.
+
         Save the file to json.
         :param full_path:
         :return:
@@ -723,6 +796,8 @@ class PandaPowerBackend(Backend):
 
     def get_line_status(self):
         """
+        .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
+
         As all the functions related to powerline, pandapower split them into multiple dataframe (some for transformers,
         some for 3 winding transformers etc.). We make sure to get them all here.
         """
@@ -732,10 +807,6 @@ class PandaPowerBackend(Backend):
         return np.concatenate((self._grid.line["in_service"].values, self._grid.trafo["in_service"].values)).astype(dt_bool)
 
     def get_line_flow(self):
-        """
-        return the powerflow in amps in all powerlines.
-        :return:
-        """
         return self.a_or
 
     def _disconnect_line(self, id_):
@@ -830,13 +901,13 @@ class PandaPowerBackend(Backend):
         return self.cst_1 * self.p_or, self.cst_1 * self.q_or, self.cst_1 * self.v_or, self.cst_1 * self.a_or
 
     def lines_ex_info(self):
-        return self.cst_1 * self.p_ex, self.cst_1 * self.q_ex,self.cst_1 * self.v_ex, self.cst_1 * self.a_ex
+        return self.cst_1 * self.p_ex, self.cst_1 * self.q_ex, self.cst_1 * self.v_ex, self.cst_1 * self.a_ex
 
     def shunt_info(self):
         shunt_p = self.cst_1 * self._grid.res_shunt["p_mw"].values.astype(dt_float)
         shunt_q = self.cst_1 * self._grid.res_shunt["q_mvar"].values.astype(dt_float)
-        shunt_v = self._grid.res_bus["vm_pu"].values[self._grid.shunt["bus"].values]
-        shunt_v *= self._grid.bus["vn_kv"].values[self._grid.shunt["bus"]]
+        shunt_v = self._grid.res_bus["vm_pu"].loc[self._grid.shunt["bus"].values].values.astype(dt_float)
+        shunt_v *= self._grid.bus["vn_kv"].loc[self._grid.shunt["bus"].values].values.astype(dt_float)
         shunt_bus = self._grid.shunt["bus"].values < self.__nb_bus_before
         shunt_bus = 1 * shunt_bus
         shunt_bus = shunt_bus.astype(dt_int)
