@@ -22,6 +22,11 @@ from grid2op.Chronics.GridValue import GridValue
 
 class GridStateFromFile(GridValue):
     """
+    .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
+
+        Do not attempt to create an object of this class. This is initialized by the environment
+        at its creation.
+
     Read the injections values from a file stored on hard drive. More detailed about the files is provided in the
     :func:`GridStateFromFile.initialize` method.
 
@@ -41,6 +46,26 @@ class GridStateFromFile(GridValue):
     In this values, only 1 episode is stored. If the end of the episode is reached and another one should start, then
     it will loop from the beginning.
 
+    It reads the following files from the "path" location specified:
+
+    - "prod_p.csv": for each time steps, this file contains the value for the active production of
+      each generators of the grid (it counts as many rows as the number of time steps - and its header)
+      and as many columns as the number of generators on the grid. The header must contains the names of
+      the generators used to map their value on the grid. Values must be convertible to floating point.
+    - "prod_v.csv": same as "prod_p.csv" but for the production voltage setpoint.
+    - "load_p.csv": same as "prod_p.csv" but for the load active value (number of columns = number of loads)
+    - "load_q.csv": same as "prod_p.csv" but for the load reactive value (number of columns = number of loads)
+    - "maintenance.csv": that contains whether or not there is a maintenance for a given powerline (column) at
+      each time step (row).
+    - "hazards.csv": that contains whether or not there is a hazard for a given powerline (column) at
+      each time step (row).
+
+    If a file is missing, it is understood as "this value will not be modified". For example, if the file
+    "prod_v.csv" is not present, it will be equivalent as not modifying the production voltage setpoint, never.
+
+    Except if the attribute :attr:`GridStateFromFile.sep` is modified, the above tables should be "semi colon" (;)
+    separated.
+
     Attributes
     ----------
     path: ``str``
@@ -85,6 +110,12 @@ class GridStateFromFile(GridValue):
                  start_datetime=datetime(year=2019, month=1, day=1),
                  chunk_size=None):
         """
+        .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
+
+        Do not attempt to create an object of this class. This is initialized by the environment
+        at its creation.
+
+
         Build an instance of GridStateFromFile. Such an instance should be built before an :class:`grid2op.Environment`
         is created.
 
@@ -290,6 +321,10 @@ class GridStateFromFile(GridValue):
     def initialize(self, order_backend_loads, order_backend_prods, order_backend_lines, order_backend_subs,
                    names_chronics_to_backend=None):
         """
+        .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
+
+            Called at the creation of the environment.
+
         In this function, the numpy arrays are read from the csv using the panda.dataframe engine.
 
         In order to be valid, the folder located at :attr:`GridStateFromFile.path` can contain:
@@ -325,10 +360,6 @@ class GridStateFromFile(GridValue):
         ----------
         See help of :func:`GridValue.initialize` for a detailed help about the parameters.
 
-        Returns
-        -------
-        ``None``
-
         """
         self.n_gen = len(order_backend_prods)
         self.n_load = len(order_backend_loads)
@@ -463,13 +494,6 @@ class GridStateFromFile(GridValue):
 
     @staticmethod
     def _file_len(fname, ext_):
-        # i = -1
-        # reading_mode = "r"
-        # if ext_ != ".csv":
-        #     reading_mode += "b"
-        # with open(fname, reading_mode) as f:
-        #     for i, l in enumerate(f):
-        #         pass
         res = pd.read_csv(fname, sep="@", dtype=str).shape[0]
         return res
 
@@ -520,6 +544,8 @@ class GridStateFromFile(GridValue):
 
     def done(self):
         """
+        .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
+
         Compare to :func:`GridValue.done` an episode can be over for 2 main reasons:
 
           - :attr:`GridValue.max_iter` has been reached
@@ -617,20 +643,6 @@ class GridStateFromFile(GridValue):
         return self.current_datetime, res, maintenance_time, maintenance_duration, hazard_duration, prod_v
 
     def check_validity(self, backend):
-        """
-        A call to this method ensure that the action that will be sent to the current :class:`grid2op.Environment`
-        can be properly implemented by its :class:`grid2op.Backend`.
-        This specific method check that the dimension of all vectors are consistent
-
-        Parameters
-        ----------
-        backend: :class:`grid2op.Backend.Backend`
-            The backend used by the :class:`grid2op.Environment.Environment`
-
-        Returns
-        -------
-        ``None``
-        """
         at_least_one = False
         if self.load_p is not None:
             if self.load_p.shape[1] != backend.n_load:
@@ -805,6 +817,28 @@ class GridStateFromFile(GridValue):
                 self._order_backend_lines, self._order_backend_lines]
 
     def split_and_save(self, datetime_beg, datetime_end, path_out):
+        """
+        You can use this function to save the values of the chronics in a format that will be loadable
+        by :class:`GridStateFromFile`
+
+        Notes
+        -----
+        Prefer using the :func:`Multifolder.split_and_save` that handles different chronics
+
+        Parameters
+        ----------
+        datetime_beg: ``str``
+            Time stamp of the beginning of the data you want to save (time stamp in "%Y-%m-%d %H:%M"
+            format)
+
+        datetime_end: ``str``
+            Time stamp of the end of the data you want to save (time stamp in "%Y-%m-%d %H:%M"
+            format)
+
+        path_out: ``str``
+            Location where to save the data
+
+        """
         # work on a copy of myself
         tmp = copy.deepcopy(self)
         datetime_beg = self._convert_datetime(datetime_beg)
@@ -857,4 +891,3 @@ class GridStateFromFile(GridValue):
         tmp_for_time_delta = datetime(year=2018, month=1, day=1, hour=0, minute=0, second=0) + self.time_interval
         with open(os.path.join(path_out, "time_interval.info"), "w") as f:
             f.write("{:%H:%M}\n".format(tmp_for_time_delta))
-
