@@ -19,9 +19,12 @@ from grid2op.Episode.EpisodeData import EpisodeData
 
 class EpisodeReplay(object):
     """
-    This class allows to see visually what an agent has done during an episode. It uses for now the "PlotPygame" as the
+    This class allows to see visually what an agent has done during an episode. It uses for now the "PlotMatplot" as the
     method to plot the different states of the system. It reads directly data from the runner.
 
+    Examples
+    --------
+
     It can be used the following manner.
 
     .. code-block:: python
