@@ -38,6 +38,7 @@ TEST_DEV_ENVS = {
     "case14_test": DEV_DATASET.format("rte_case14_test"),
     "case5_example": DEV_DATASET.format("rte_case5_example"),
     "case14_fromfile": DEV_DATASET.format("rte_case14_test"),
+    "educ_case14_redisp": DEV_DATASET.format("educ_case14_redisp"),
 }
 
 _REQUEST_FAIL_EXHAUSTED_ERR = "Impossible to retrieve data at \"{}\".\n" \
@@ -48,8 +49,44 @@ _REQUEST_FAIL_RETRY_ERR = "Failure to get a reponse from the url \"{}\".\n" \
 _REQUEST_EXCEPT_RETRY_ERR = "Exception in getting an answer from \"{}\".\n" \
                             "Retrying.. {} attempt(s) remaining"
 
+_LIST_REMOTE_URL = "https://api.github.com/repos/bdonnot/grid2op-datasets/contents/datasets.json"
+_LIST_REMOTE_KEY = "download_url"
+_LIST_REMOTE_INVALID_CONTENT_JSON_ERR = "Impossible to retrieve available datasets. " \
+                                        "File could not be converted to json. " \
+                                        "Parsing error:\n {}"
+_LIST_REMOTE_CORRUPTED_CONTENT_JSON_ERR = "Corrupted json retrieved from github api. " \
+                                         "Please wait a few minutes and try again. " \
+                                         "If the error persist, contact grid2op organizers"
+_LIST_REMOTE_INVALID_DATASETS_JSON_ERR = "Impossible to retrieve available datasets. " \
+                                         "File could not be converted to json. " \
+                                         "The error was \n\"{}\""
+
+_FETCH_ENV_UNKNOWN_ERR = "Impossible to find the environment named \"{}\".\n" \
+                         "Current available environments are:\n{}"
+
+_MULTIMIX_FILE = ".multimix"
+
+_MAKE_DEV_ENV_WARN = "You are using a development environment. " \
+                     "This environment is not intended for training agents. It might not be up to date "\
+                     "and its primary use if for tests (hence the \"test=True\" you passed as argument). "\
+                     "Use at your own risk."
+_MAKE_DEV_ENV_DEPRECATED_WARN = "Dev env \"{}\" has been deprecated " \
+                                "and will be removed in future version.\n" \
+                                "Please update to dev envs starting by \"rte\" or \"l2rpn\""
+_MAKE_FIRST_TIME_WARN = "It is the first time you use the environment \"{}\".\n" \
+                        "We will attempt to download this environment from remote"
+_MAKE_UNKNOWN_ENV = "Impossible to load the environment named \"{}\"."
+
+_EXTRACT_DS_NAME_CONVERT_ERR = "The \"dataset_name\" argument " \
+                               "should be convertible to string, " \
+                               "but \"{}\" was provided."
+_EXTRACT_DS_NAME_RECO_ERR = "Impossible to recognize the environment name from path \"{}\""
+
 
 def _send_request_retry(url, nb_retry=10, gh_session=None):
+    """
+    .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
+    """
     if nb_retry <= 0:
         raise Grid2OpException(_REQUEST_FAIL_EXHAUSTED_ERR.format(url))
 
@@ -73,24 +110,6 @@ def _send_request_retry(url, nb_retry=10, gh_session=None):
         return _send_request_retry(url, nb_retry=nb_retry-1, gh_session=gh_session)
 
 
-# _LIST_REMOTE_URL = "https://api.github.com/repos/bdonnot/grid2op-datasets/contents/contents.json"
-_LIST_REMOTE_URL = "https://api.github.com/repos/bdonnot/grid2op-datasets/contents/datasets.json"
-_LIST_REMOTE_KEY = "download_url"
-_LIST_REMOTE_INVALID_CONTENT_JSON_ERR = "Impossible to retrieve available datasets. " \
-                                        "File could not be converted to json. " \
-                                        "Parsing error:\n {}"
-_LIST_REMOTE_CORRUPTED_CONTENT_JSON_ERR = "Corrupted json retrieved from github api. " \
-                                         "Please wait a few minutes and try again. " \
-                                         "If the error persist, contact grid2op organizers"
-_LIST_REMOTE_INVALID_DATASETS_JSON_ERR = "Impossible to retrieve available datasets. " \
-                                         "File could not be converted to json. " \
-                                         "The error was \n\"{}\""
-
-_FETCH_ENV_UNKNOWN_ERR = "Impossible to find the environment named \"{}\".\n" \
-                         "Current available environments are:\n{}"
-# _FETCH_ENV_TAR_URL = "https://github.com/BDonnot/grid2op-datasets/releases/download/{}/{}.tar.bz2"
-
-
 def _retrieve_github_content(url, is_json=True):
     answer = _send_request_retry(url)
     try:
@@ -130,12 +149,6 @@ def _fecth_environments(dataset_name):
     return url, ds_name_dl
 
 
-_EXTRACT_DS_NAME_CONVERT_ERR = "The \"dataset_name\" argument " \
-                               "should be convertible to string, " \
-                               "but \"{}\" was provided."
-_EXTRACT_DS_NAME_RECO_ERR = "Impossible to recognize the environment name from path \"{}\""
-
-
 def _extract_ds_name(dataset_path):
     """
     If a path is provided, clean it to have a proper datasetname.
@@ -167,32 +180,21 @@ def _extract_ds_name(dataset_path):
     dataset_name = os.path.splitext(dataset_name)[0]
     return dataset_name
 
-_MULTIMIX_FILE = ".multimix"
 
 def _aux_is_multimix(dataset_path):
     if os.path.exists(os.path.join(dataset_path, _MULTIMIX_FILE)):
         return True
     return False
 
+
 def _aux_make_multimix(dataset_path, **kwargs):
     # Local import to prevent imports loop
     from grid2op.Environment import MultiMixEnvironment
 
     return MultiMixEnvironment(dataset_path, **kwargs)
 
-_MAKE_DEV_ENV_WARN = "You are using a development environment. " \
-                     "This environment is not intended for training agents. It might not be up to date "\
-                     "and its primary use if for tests (hence the \"test=True\" you passed as argument). "\
-                     "Use at your own risk."
-_MAKE_DEV_ENV_DEPRECATED_WARN = "Dev env \"{}\" has been deprecated " \
-                                "and will be removed in future version.\n" \
-                                "Please update to dev envs starting by \"rte\" or \"l2rpn\""
-_MAKE_FIRST_TIME_WARN = "It is the first time you use the environment \"{}\".\n" \
-                        "We will attempt to download this environment from remote"
-_MAKE_UNKNOWN_ENV = "Impossible to load the environment named \"{}\"."
 
-
-def make(dataset="rte_case14_realistic", test=False, **kwargs):
+def make(dataset="rte_case14_realistic", test=False, _add_to_name="", **kwargs):
     """
     This function is a shortcut to rapidly create some (pre defined) environments within the grid2op Framework.
 
@@ -214,6 +216,9 @@ def make(dataset="rte_case14_realistic", test=False, **kwargs):
         Other keyword argument to give more control on the environment you are creating. See
         the Parameters information of the :func:`make_from_dataset_path`.
 
+    _add_to_name:
+        Internal, do not use
+
     Returns
     -------
     env: :class:`grid2op.Environment.Environment`
@@ -263,14 +268,14 @@ def make(dataset="rte_case14_realistic", test=False, **kwargs):
     if test:
         warnings.warn(_MAKE_DEV_ENV_WARN)
         # Warning for deprecated dev envs
-        if not (dataset_name.startswith("rte") or dataset_name.startswith("l2rpn")):
+        if not (dataset_name.startswith("rte") or dataset_name.startswith("l2rpn") or dataset_name.startswith("educ")):
             warnings.warn(_MAKE_DEV_ENV_DEPRECATED_WARN.format(dataset_name))
         ds_path = TEST_DEV_ENVS[dataset_name]
         # Check if multimix from path
         if _aux_is_multimix(ds_path):
             make_from_path_fn = _aux_make_multimix
 
-        return make_from_path_fn(dataset_path=ds_path, **kwargs)
+        return make_from_path_fn(dataset_path=ds_path, _add_to_name=_add_to_name, **kwargs)
 
     # Env directory is present in the DEFAULT_PATH_DATA
     if os.path.exists(real_ds_path):
@@ -288,4 +293,3 @@ def make(dataset="rte_case14_realistic", test=False, **kwargs):
     if _aux_is_multimix(real_ds_path):
         make_from_path_fn = _aux_make_multimix
     return make_from_path_fn(dataset_path=real_ds_path, **kwargs)
-
