@@ -10,6 +10,7 @@ from grid2op.Exceptions import OpponentError
 
 class OpponentSpace(object):
     """
+    Is similar to the action space, but for the opponent.
 
     Attributes
     ----------
@@ -63,12 +64,12 @@ class OpponentSpace(object):
         #    raise OpponentError("Impossible to build an opponent reward with a reward of type {}".format(opponent_reward_class))
         # self.opp_reward_helper = RewardHelper(opponent_reward_class)
 
-    def init_opponent(self, **kwargs):
+    def init_opponent(self, partial_env, **kwargs):
         """
         Generic function used to initialize the opponent. For example, if an opponent reads from a file, the
         path where is the file is located should be pass with this method.
         """
-        self.opponent.init(**kwargs)
+        self.opponent.init(partial_env=partial_env, **kwargs)
 
     def reset(self):
         """
