@@ -13,6 +13,11 @@ from grid2op.Exceptions import OpponentError
 
 
 class RandomLineOpponent(BaseOpponent):
+    """
+    An opponent that disconnect at random any powerlines among a specified list given
+    at the initialization.
+
+    """
     def __init__(self, action_space):
         BaseOpponent.__init__(self, action_space)
         self._do_nothing = None
@@ -22,7 +27,21 @@ class RandomLineOpponent(BaseOpponent):
         # this is the constructor:
         # it should have the exact same signature as here
 
-    def init(self, lines_attacked=[], **kwargs):
+    def init(self, partial_env, lines_attacked=[], **kwargs):
+        """
+        .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
+            Used when the opponent is created.
+
+        Parameters
+        ----------
+        partial_env
+        lines_attacked
+        kwargs
+
+        Returns
+        -------
+
+        """
         # this if the function used to properly set the object.
         # It has the generic signature above,
         # and it's way more flexible that the other one.
@@ -98,7 +117,7 @@ class RandomLineOpponent(BaseOpponent):
         status = observation.line_status[self._lines_ids]
 
         # If all attackable lines are disconnected
-        if np.all(status == False):
+        if np.all(~status):
             return None  # i choose not to attack in this case
 
         # Pick a line among the connected lines
