@@ -14,9 +14,9 @@ from grid2op.dtypes import dt_int, dt_float, dt_bool
 
 class Parameters:
     """
-    Main classes representing the _parameters of the game. The main paratemeters are describe bellow.
+    Main classes representing the parameters of the game. The main parameters are described bellow.
 
-    Note that changing the values of these _parameters might not be enough. If these _parameters are not used in the
+    Note that changing the values of these parameters might not be enough. If these _parameters are not used in the
     :class:`grid2op.Rules.RulesChecker`, then modifying them will have no impact at all.
 
     Attributes
@@ -62,6 +62,7 @@ class Parameters:
         "alternative current" powerflow. It is also less precise. The default is ``False``
 
     FORECAST_DC: ``bool``
+        DEPRECATED. Please use the "change_forecast_param" function of the environment
         Whether to use the direct current approximation in the :func:`grid2op.Observation.BaseObservation.simulate`
         method. Default is ``False``. Setting :attr:`FORECAST_DC` to `True` can speed up the computation of the
         `simulate` function, but will make the results less accurate.
@@ -111,7 +112,7 @@ class Parameters:
         self.ENV_DC = False
 
         # same as above, but for the forecast states
-        self.FORECAST_DC = False
+        self.FORECAST_DC = False  # DEPRECATED use "change_forecast_parameters(new_param)" with "new_param.ENV_DC=..."
 
         # maximum number of substations that can be change in one action
         self.MAX_SUB_CHANGED = dt_int(1)
@@ -155,7 +156,7 @@ class Parameters:
 
     def init_from_dict(self, dict_):
         """
-        Initialize the object given a dictionary. All keys are optional. If a key is not present in the dictionnary,
+        Initialize the object given a dictionary. All keys are optional. If a key is not present in the dictionary,
         the default parameters is used.
 
         Parameters
@@ -188,7 +189,12 @@ class Parameters:
             self.ENV_DC = Parameters._isok_txt(dict_["ENV_DC"])
 
         if "FORECAST_DC" in dict_:
-            self.FORECAST_DC = Parameters._isok_txt(dict_["FORECAST_DC"])
+            new_val = Parameters._isok_txt(dict_["FORECAST_DC"])
+            if new_val != self.FORECAST_DC:
+                warnings.warn("The FORECAST_DC attributes is deprecated. Please change the parameters of the "
+                              "\"forecast\" backend with \"env.change_forecast_parameters(new_param)\" function "
+                              "with \"new_param.ENV_DC=...\" ")
+            self.FORECAST_DC = new_val
 
         if "MAX_SUB_CHANGED" in dict_:
             self.MAX_SUB_CHANGED = dt_int(dict_["MAX_SUB_CHANGED"])
@@ -263,7 +269,7 @@ class Parameters:
     @staticmethod
     def from_json(json_path):
         """
-        Create instance from a json path.
+        Create instance of a Parameters from a path where is a json is saved.
 
         Parameters
         ----------
