@@ -6,8 +6,10 @@
 # SPDX-License-Identifier: MPL-2.0
 # This file is part of Grid2Op, Grid2Op a testbed platform to model sequential decision making in power systems.
 
-import numpy as np
 import copy
+import warnings
+
+import numpy as np
 from abc import ABC, abstractmethod
 
 from grid2op.Observation import BaseObservation
@@ -16,8 +18,11 @@ from grid2op.PlotGrid.LayoutUtil import layout_obs_sub_load_and_gen
 from grid2op.PlotGrid.PlotUtil import PlotUtil as pltu
 from grid2op.dtypes import dt_float, dt_int
 
+
 class BasePlot(ABC):
     """
+    .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
+
     Abstract interface to plot the state of the powergrid
 
     Implement the interface with a plotting library to generate drawings 
@@ -170,6 +175,8 @@ class BasePlot(ABC):
                         sub_id, sub_name,
                         pos_x, pos_y):
         """
+        .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
+
         Update a substation into the figure
         """
         pass
@@ -218,6 +225,8 @@ class BasePlot(ABC):
                     pos_x, pos_y,
                     sub_x, sub_y):
         """
+        .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
+
         Update a load into the figure
         """
         pass
@@ -267,6 +276,8 @@ class BasePlot(ABC):
                    pos_x, pos_y,
                    sub_x, sub_y):
         """
+        .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
+
         Updates a generator into the figure
         """
         pass
@@ -319,6 +330,8 @@ class BasePlot(ABC):
                          or_bus, pos_or_x, pos_or_y,
                          ex_bus, pos_ex_x, pos_ex_y):
         """
+        .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
+
         Draws a powerline into the figure
         """
         pass
@@ -340,6 +353,8 @@ class BasePlot(ABC):
     
     def update_legend(self, figure, observation):
         """
+        .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
+
         Updates the legend for the given figure.
         """
         pass
@@ -402,7 +417,9 @@ class BasePlot(ABC):
                 continue
             gen_value = None
             if gen_values is not None:
-                gen_value = np.round(float(gen_values[gen_idx]), 2)
+                gen_value = gen_values[gen_idx]
+                if gen_value is not None:
+                    gen_value = np.round(float(gen_values[gen_idx]), 2)
             gen_x = self._grid_layout[gen_name][0]
             gen_y = self._grid_layout[gen_name][1]
             gen_subid = observation.gen_to_subid[gen_idx]
@@ -490,7 +507,8 @@ class BasePlot(ABC):
         return self.plot_info(observation=self.observation_space,
                               figure=None, redraw=True)
         
-    def plot_obs(self, observation,
+    def plot_obs(self,
+                 observation,
                  figure=None,
                  redraw=True,
                  line_info="rho",
@@ -527,6 +545,8 @@ class BasePlot(ABC):
         res: ``object``
             The figure updated with the data from the new observation.
         """
+        # TODO add gen_info=dispatch and gen_info=target_dispatch
+        # TODO add line_info=cooldown
 
         # Start by checking arguments are valid
         if not isinstance(observation, BaseObservation):            
@@ -591,7 +611,8 @@ class BasePlot(ABC):
                   load_unit="",
                   gen_values=None,
                   gen_unit="",
-                  observation=None):
+                  observation=None,
+                  coloring=None):
         """
         Plot an observation with custom values
         
@@ -625,6 +646,37 @@ class BasePlot(ABC):
         observation: :class:`grid2op.Observation.BaseObservation`
             An observation to plot, can be None if no values are drawn from the observation
 
+        coloring:
+            ``None`` for no special coloring, or "line" to color the powerline based on the value
+            ("gen" and "load" coming soon)
+
+        Examples
+        --------
+
+        More examples on how to use this function is given in the "8_PlottingCapabilities.ipynb" notebook.
+
+        The basic concept is:
+
+        .. code-block:: python
+
+            import grid2op
+            from grid2op.PlotGrid import PlotMatplot
+            env = grid2op.make()
+            plot_helper = PlotMatplot(env.observation_space)
+
+
+            # plot the layout (position of each elements) of the powergrid
+            plot_helper.plot_layout()
+
+            # project some data on the grid
+            line_values = env.get_thermal_limit()
+            plot_helper.plot_info(line_values=line_values)
+
+            # to plot an observation
+            obs = env.reset()
+            plot_helper.plot_obs(obs)
+
+
         Returns
         -------
         res: ``object``
@@ -633,13 +685,16 @@ class BasePlot(ABC):
         # Check values are in the correct format
         if line_values is not None and len(line_values) != self.observation_space.n_line:
             raise PlotError("Impossible to display these values on the powerlines: there are {} values"
-                            "provided for {} powerlines in the grid".format(len(line_values), self.observation_space.n_line))
+                            "provided for {} powerlines in the grid".format(len(line_values),
+                                                                            self.observation_space.n_line))
         if load_values is not None and len(load_values) != self.observation_space.n_load:
             raise PlotError("Impossible to display these values on the loads: there are {} values"
-                            "provided for {} loads in the grid".format(len(load_values), self.observation_space.n_load))
+                            "provided for {} loads in the grid".format(len(load_values),
+                                                                       self.observation_space.n_load))
         if gen_values is not None and len(gen_values) != self.observation_space.n_gen:
             raise PlotError("Impossible to display these values on the generators: there are {} values"
-                            "provided for {} generators in the grid".format(len(gen_values), self.observation_space.n_gen))
+                            "provided for {} generators in the grid".format(len(gen_values),
+                                                                            self.observation_space.n_gen))
 
         # Get a valid figure to draw into
         if figure is None:
@@ -655,7 +710,47 @@ class BasePlot(ABC):
         if observation is None:
             # See dummy data added in the constructor
             observation = self.observation_space
-            
+            if coloring is not None:
+                observation = copy.deepcopy(observation)
+                if coloring == "line":
+                    if line_values is None:
+                        raise PlotError("Impossible to color the grid based on the line information (key word argument "
+                                        "\"line_values\") if this argument is None.")
+
+                    observation.rho = copy.deepcopy(line_values)
+                    try:
+                        observation.rho = np.array(observation.rho).astype(np.float)
+                    except:
+                        raise PlotError("Impossible to convert the input values (line_values) to floating point")
+
+                    # rescaling to have range 0 - 1.0
+                    tmp = observation.rho[np.isfinite(observation.rho)]
+                    observation.rho -= (np.min(tmp) - 1e-1)  # so the min is 1e-1 otherwise 0.0 is plotted as black
+                    tmp = observation.rho[np.isfinite(observation.rho)]
+                    observation.rho /= np.max(tmp)
+                elif coloring == "load":
+                    # TODO
+                    warnings.warn("ooloring = loads is not available at the moment")
+                elif coloring == "gen":
+                    if gen_values is None:
+                        raise PlotError("Impossible to color the grid based on the gen information (key word argument "
+                                        "\"gen_values\") if this argument is None.")
+
+                    observation.prod_p = copy.deepcopy(gen_values)
+                    try:
+                        observation.prod_p = np.array(observation.prod_p).astype(np.float)
+                    except:
+                        raise PlotError("Impossible to convert the input values (gen_values) to floating point")
+
+                    # rescaling to have range 0 - 1.0
+                    tmp = observation.prod_p[np.isfinite(observation.prod_p)]
+                    if np.any(np.isfinite(observation.prod_p)):
+                        observation.prod_p -= (np.min(tmp) - 1e-1)  # so the min is 1e-1 otherwise 0.0 is plotted as black
+                        tmp = observation.prod_p[np.isfinite(observation.prod_p)]
+                        observation.prod_p /= np.max(tmp)
+                else:
+                    raise PlotError("coloring must be one of \"line\", \"load\" or \"gen\"")
+
         # Trigger draw calls
         self._plot_lines(fig, observation, line_values, line_unit, redraw)
         self._plot_loads(fig, observation, load_values, load_unit, redraw)
