@@ -9,8 +9,13 @@
 from grid2op.Reward.BaseReward import BaseReward
 from grid2op.Reward.ConstantReward import ConstantReward
 
+
 class RewardHelper:
     """
+    .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
+        It is a class internal to the :class:`grid2op.Environment.Environment` do not use outside
+        of its purpose and do not attempt to modify it.
+
     This class aims at making the creation of rewards class more automatic by the :class:`grid2op.Environment`.
 
     It is not recommended to derived or modified this class. If a different reward need to be used, it is recommended
@@ -59,7 +64,6 @@ class RewardHelper:
         Gives the reward that follows the execution of the :class:`grid2op.BaseAction.BaseAction` action in the
         :class:`grid2op.Environment.Environment` env;
 
-
         Parameters
         ----------
         action: :class:`grid2op.Action.Action`
@@ -85,11 +89,9 @@ class RewardHelper:
 
         Returns
         -------
+        res: ``float``
+            The computed reward
 
         """
-        if not has_error:
-            res = self.template_reward(action, env, has_error, is_done, is_illegal, is_ambiguous)
-        else:
-            # no more data to consider, no powerflow has been run, reward is what it is
-            res = self.template_reward.reward_min
+        res = self.template_reward(action, env, has_error, is_done, is_illegal, is_ambiguous)
         return res
