@@ -6,4 +6,4 @@ __all__ = [
 
 from grid2op.Space.RandomObject import RandomObject
 from grid2op.Space.SerializableSpace import SerializableSpace
-from grid2op.Space.Space import GridObjects
+from grid2op.Space.GridObjects import GridObjects
