@@ -67,6 +67,8 @@ class BaseTestRedispatch(MakeBackend):
                                actionClass=BaseAction,
                                name="test_redisp_env1")
         self.array_double_dispatch = np.array([0.,  10.,  20.,   0., -30.])
+        # self.array_double_dispatch = np.array([0.,  11.208119,  12.846733, 0., -24.054852])
+        self.tol_one = self.env._tol_poly
 
     def tearDown(self):
         self.env.close()
@@ -75,8 +77,8 @@ class BaseTestRedispatch(MakeBackend):
         self.skip_if_needed()
         act = self.env.action_space({"redispatch": [(1, -10)]})
         obs, reward, done, info = self.env.step(act)
-        assert np.all(obs.prod_p >= self.env.gen_pmin)
-        assert np.all(obs.prod_p <= self.env.gen_pmax)
+        assert np.all(obs.prod_p - self.env.gen_pmin >= -self.tol_one)
+        assert np.all(obs.prod_p <= self.env.gen_pmax + self.tol_one)
         assert np.abs(np.sum(obs.actual_dispatch)) <= self.tol_one
 
     def test_no_impact_env(self):
@@ -92,10 +94,10 @@ class BaseTestRedispatch(MakeBackend):
         obs, reward, done, info = self.env.step(act)
         assert self.compare_vect(obsinit.prod_p, ref_data)
 
-        target_val = obs.prod_p + self.env.actual_dispatch
+        target_val = obs.prod_p + self.env._actual_dispatch
         assert self.compare_vect(obs.prod_p[:-1], target_val[:-1])  # I remove last component which is the slack bus
-        assert np.all(obs.prod_p >= self.env.gen_pmin)
-        assert np.all(target_val <= self.env.gen_pmax)
+        assert np.all(obs.prod_p - self.env.gen_pmin >= -self.tol_one)
+        assert np.all(target_val <= self.env.gen_pmax + self.tol_one)
         assert np.all(obs.prod_p - obsinit.prod_p <= self.env.gen_max_ramp_up)
         assert np.all(obsinit.prod_p - obs.prod_p <= self.env.gen_max_ramp_down)
 
@@ -104,17 +106,18 @@ class BaseTestRedispatch(MakeBackend):
         self.skip_if_needed()
         act = self.env.action_space({"redispatch": [2, 5]})
         obs, reward, done, info = self.env.step(act)
-        assert np.abs(np.sum(self.env.actual_dispatch)) <= self.tol_one
+        assert np.abs(np.sum(self.env._actual_dispatch)) <= self.tol_one
         th_dispatch = np.array([ 0. , -2.5,  5. ,  0. , -2.5])
-        assert self.compare_vect(self.env.actual_dispatch, th_dispatch)
-        target_val = self.chronics_handler.real_data.prod_p[1, :] + self.env.actual_dispatch
+        th_dispatch = np.array([0., -1.4814819,  5.,  0., -3.518518])
+        assert self.compare_vect(self.env._actual_dispatch, th_dispatch)
+        target_val = self.chronics_handler.real_data.prod_p[1, :] + self.env._actual_dispatch
         assert self.compare_vect(obs.prod_p[:-1], target_val[:-1])  # I remove last component which is the slack bus
-        assert np.all(obs.prod_p >= self.env.gen_pmin)
-        assert np.all(target_val <= self.env.gen_pmax)
+        assert np.all(obs.prod_p - self.env.gen_pmin >= -self.tol_one)
+        assert np.all(target_val <= self.env.gen_pmax + self.tol_one)
 
         # check that the redispatching is apply in the right direction
-        indx_ok = self.env.target_dispatch != 0.
-        assert np.all(np.sign(self.env.actual_dispatch[indx_ok]) == np.sign(self.env.target_dispatch[indx_ok]))
+        indx_ok = self.env._target_dispatch != 0.
+        assert np.all(np.sign(self.env._actual_dispatch[indx_ok]) == np.sign(self.env._target_dispatch[indx_ok]))
 
     def test_redispatch_act_above_pmax(self):
         # in this test, the asked redispatching for generator 2 would make it above pmax, so the environment
@@ -122,13 +125,15 @@ class BaseTestRedispatch(MakeBackend):
         self.skip_if_needed()
         act = self.env.action_space({"redispatch": [2, 60]})
         obs, reward, done, info = self.env.step(act)
-        assert np.abs(np.sum(self.env.actual_dispatch)) <= self.tol_one
-        th_dispatch = np.array([0.0000000e+00, -2.3289999e+01,  5.0890003e+01,  9.9999998e-03, -2.7610004e+01])
-        assert self.compare_vect(self.env.actual_dispatch, th_dispatch)
-        target_val = self.chronics_handler.real_data.prod_p[1, :] + self.env.actual_dispatch
+        assert np.abs(np.sum(self.env._actual_dispatch)) <= self.tol_one
+        th_dispatch = np.array([  0.      , -23.2999  ,  50.899902,   0.      , -27.600002])
+        th_dispatch = np.array([0., -20., 40., 0., -20.])
+        th_dispatch = np.array([  0.      , -13.227808,  50.90005 ,   0.      , -37.67224 ])
+        assert self.compare_vect(self.env._actual_dispatch, th_dispatch)
+        target_val = self.chronics_handler.real_data.prod_p[1, :] + self.env._actual_dispatch
         assert self.compare_vect(obs.prod_p[:-1], target_val[:-1])  # I remove last component which is the slack bus
-        assert np.all(obs.prod_p >= self.env.gen_pmin)
-        assert np.all(target_val <= self.env.gen_pmax)
+        assert np.all(obs.prod_p - self.env.gen_pmin >= -self.tol_one)
+        assert np.all(target_val <= self.env.gen_pmax + self.tol_one)
 
     def test_two_redispatch_act(self):
         self.skip_if_needed()
@@ -138,19 +143,19 @@ class BaseTestRedispatch(MakeBackend):
         obs, reward, done, info = self.env.step(act)
         th_dispatch = np.array([0., 10, 20., 0., 0.])
         th_dispatch[1] += obs_first.actual_dispatch[1]
-        assert self.compare_vect(self.env.target_dispatch, th_dispatch)
+        assert self.compare_vect(self.env._target_dispatch, th_dispatch)
         # check that the redispatching is apply in the right direction
-        indx_ok = self.env.target_dispatch != 0.
-        assert np.all(np.sign(self.env.actual_dispatch[indx_ok]) == np.sign(self.env.target_dispatch[indx_ok]))
+        indx_ok = self.env._target_dispatch != 0.
+        assert np.all(np.sign(self.env._actual_dispatch[indx_ok]) == np.sign(self.env._target_dispatch[indx_ok]))
         th_dispatch = np.array([0.,  10.,  20.,   0., -30.])
-        th_dispatch = np.array([ 0.0000000e+00, -5.0001135e-03, 2.0000000e+01,  9.9999998e-03, -2.0005001e+01])
-        assert self.compare_vect(self.env.actual_dispatch, th_dispatch)
+        th_dispatch = np.array([0.,   4.0765514,  20.004545,   0., -24.081097])
+        assert self.compare_vect(self.env._actual_dispatch, th_dispatch)
 
-        target_val = self.chronics_handler.real_data.prod_p[2, :] + self.env.actual_dispatch
+        target_val = self.chronics_handler.real_data.prod_p[2, :] + self.env._actual_dispatch
         assert self.compare_vect(obs.prod_p[:-1], target_val[:-1])  # I remove last component which is the slack bus
-        assert np.abs(np.sum(self.env.actual_dispatch)) <= self.tol_one
-        assert np.all(target_val <= self.env.gen_pmax)
-        assert np.all(obs.prod_p >= self.env.gen_pmin)
+        assert np.abs(np.sum(self.env._actual_dispatch)) <= self.tol_one
+        assert np.all(target_val <= self.env.gen_pmax + self.tol_one)
+        assert np.all(obs.prod_p - self.env.gen_pmin >= -self.tol_one)
 
     def test_redispacth_two_gen(self):
         self.skip_if_needed()
@@ -158,14 +163,14 @@ class BaseTestRedispatch(MakeBackend):
         obs, reward, done, info = self.env.step(act)
         assert not done
         th_dispatch = np.array([0., 10, 20., 0., 0.])
-        assert self.compare_vect(self.env.target_dispatch, th_dispatch)
-        assert self.compare_vect(self.env.actual_dispatch, self.array_double_dispatch)
+        assert self.compare_vect(self.env._target_dispatch, th_dispatch)
+        assert self.compare_vect(self.env._actual_dispatch, self.array_double_dispatch)
 
         # check that the redispatching is apply in the right direction
-        indx_ok = self.env.target_dispatch != 0.
-        assert np.all(np.sign(self.env.actual_dispatch[indx_ok]) == np.sign(self.env.target_dispatch[indx_ok]))
-        assert np.all(obs.prod_p <= self.env.gen_pmax)
-        assert np.all(obs.prod_p >= self.env.gen_pmin)
+        indx_ok = self.env._target_dispatch != 0.
+        assert np.all(np.sign(self.env._actual_dispatch[indx_ok]) == np.sign(self.env._target_dispatch[indx_ok]))
+        assert np.all(obs.prod_p <= self.env.gen_pmax + self.tol_one)
+        assert np.all(obs.prod_p - self.env.gen_pmin >= -self.tol_one)
 
     def test_redispacth_all_gen(self):
         # this should be exactly the same as the previous one
@@ -174,14 +179,14 @@ class BaseTestRedispatch(MakeBackend):
         obs, reward, done, info = self.env.step(act)
 
         th_dispatch = np.array([0., 10, 20., 0., -30.])
-        assert self.compare_vect(self.env.target_dispatch, th_dispatch)
-        assert self.compare_vect(self.env.actual_dispatch, self.array_double_dispatch)
+        assert self.compare_vect(self.env._target_dispatch, th_dispatch)
+        assert self.compare_vect(self.env._actual_dispatch, self.array_double_dispatch)
 
         # check that the redispatching is apply in the right direction
-        indx_ok = self.env.target_dispatch != 0.
-        assert np.all(np.sign(self.env.actual_dispatch[indx_ok]) == np.sign(self.env.target_dispatch[indx_ok]))
-        assert np.all(obs.prod_p <= self.env.gen_pmax)
-        assert np.all(obs.prod_p >= self.env.gen_pmin)
+        indx_ok = self.env._target_dispatch != 0.
+        assert np.all(np.sign(self.env._actual_dispatch[indx_ok]) == np.sign(self.env._target_dispatch[indx_ok]))
+        assert np.all(obs.prod_p <= self.env.gen_pmax + self.tol_one)
+        assert np.all(obs.prod_p - self.env.gen_pmin >= -self.tol_one)
 
     def test_count_turned_on(self):
         self.skip_if_needed()
@@ -189,45 +194,46 @@ class BaseTestRedispatch(MakeBackend):
 
         # recoded it: it's the normal behavior to call "env.reset()" to get the first time step
         obs = self.env.reset()
-        assert np.all(self.env.gen_uptime == np.array([0, 1, 1, 0, 1]))
-        assert np.all(self.env.gen_downtime == np.array([1, 0, 0, 1, 0]))
-        assert np.all(obs.prod_p <= self.env.gen_pmax)
-        assert np.all(obs.prod_p >= self.env.gen_pmin)
+        assert np.all(self.env._gen_uptime == np.array([0, 1, 1, 0, 1]))
+        assert np.all(self.env._gen_downtime == np.array([1, 0, 0, 1, 0]))
+        assert np.all(obs.prod_p <= self.env.gen_pmax + self.tol_one)
+        assert np.all(obs.prod_p - self.env.gen_pmin >= -self.tol_one)
 
         obs, reward, done, info = self.env.step(act)
-        assert np.all(self.env.gen_uptime == np.array([0, 2, 2, 0, 2]))
-        assert np.all(self.env.gen_downtime == np.array([2, 0, 0, 2, 0]))
-        assert np.all(obs.prod_p <= self.env.gen_pmax)
-        assert np.all(obs.prod_p >= self.env.gen_pmin)
+        assert np.all(self.env._gen_uptime == np.array([0, 2, 2, 0, 2]))
+        assert np.all(self.env._gen_downtime == np.array([2, 0, 0, 2, 0]))
+        assert np.all(obs.prod_p <= self.env.gen_pmax + self.tol_one)
+        assert np.all(obs.prod_p - self.env.gen_pmin >= -self.tol_one)
 
         for i in range(64):
             obs, reward, done, info = self.env.step(act)
-            assert np.all(obs.prod_p <= self.env.gen_pmax)
-            assert np.all(obs.prod_p >= self.env.gen_pmin)
+            assert np.all(obs.prod_p <= self.env.gen_pmax + self.tol_one)
+            assert np.all(obs.prod_p - self.env.gen_pmin >= -self.tol_one)
 
         obs, reward, done, info = self.env.step(act)
-        assert np.all(self.env.gen_uptime == np.array([0, 67, 67,  1, 67]))
-        assert np.all(self.env.gen_downtime == np.array([67, 0, 0, 0, 0]))
-        assert np.all(obs.prod_p <= self.env.gen_pmax)
-        assert np.all(obs.prod_p >= self.env.gen_pmin)
+        assert np.all(self.env._gen_uptime == np.array([0, 67, 67, 1, 67]))
+        assert np.all(self.env._gen_downtime == np.array([67, 0, 0, 0, 0]))
+        assert np.all(obs.prod_p <= self.env.gen_pmax + self.tol_one)
+        assert np.all(obs.prod_p - self.env.gen_pmin >= -self.tol_one)
 
         obs, reward, done, info = self.env.step(act)
-        assert np.all(self.env.gen_uptime == np.array([1, 68, 68,  2, 68]))
-        assert np.all(self.env.gen_downtime == np.array([0, 0, 0, 0, 0]))
-        assert np.all(obs.prod_p <= self.env.gen_pmax)
-        assert np.all(obs.prod_p >= self.env.gen_pmin)
+        assert np.all(self.env._gen_uptime == np.array([1, 68, 68, 2, 68]))
+        assert np.all(self.env._gen_downtime == np.array([0, 0, 0, 0, 0]))
+        assert np.all(obs.prod_p <= self.env.gen_pmax + self.tol_one)
+        assert np.all(obs.prod_p - self.env.gen_pmin >= -self.tol_one)
 
     def test_redispacth_twice_same(self):
         self.skip_if_needed()
         # this should be exactly the same as the previous one
         act = self.env.action_space({"redispatch": [(2, 5.)]})
         obs, reward, done, info = self.env.step(act)
-        assert np.all(obs.target_dispatch == np.array([ 0.,  0., 5.,  0.,  0.]))
+        assert np.all(obs.target_dispatch == np.array([0.,  0., 5.,  0.,  0.]))
         assert np.abs(np.sum(obs.actual_dispatch)) <= self.tol_one
-        th_disp = np.array([ 0. , -2.5,  5. ,  0. , -2.5])
+        th_disp = np.array([0., -2.5,  5.,  0., -2.5])
+        th_disp = np.array([ 0.       , -1.4814819,  5.       ,  0.       , -3.518518 ])
         assert self.compare_vect(obs.actual_dispatch, th_disp)
-        assert np.all(obs.prod_p <= self.env.gen_pmax)
-        assert np.all(obs.prod_p >= self.env.gen_pmin)
+        assert np.all(obs.prod_p <= self.env.gen_pmax + self.tol_one)
+        assert np.all(obs.prod_p - self.env.gen_pmin >= -self.tol_one)
 
         act = self.env.action_space({"redispatch": [(2, 5.)]})
         obs, reward, done, info = self.env.step(act)
@@ -235,27 +241,31 @@ class BaseTestRedispatch(MakeBackend):
         assert np.abs(np.sum(obs.actual_dispatch)) <= self.tol_one
         th_disp = np.array([ 0., -5., 10.,  0., -5.])
         assert self.compare_vect(obs.actual_dispatch, th_disp)
-        assert np.all(obs.prod_p <= self.env.gen_pmax)
-        assert np.all(obs.prod_p >= self.env.gen_pmin)
+        assert np.all(obs.prod_p <= self.env.gen_pmax + self.tol_one)
+        assert np.all(obs.prod_p - self.env.gen_pmin >= -self.tol_one)
 
     def test_redispacth_secondabovepmax(self):
         self.skip_if_needed()
         act = self.env.action_space({"redispatch": [(2, 20.)]})
-        obs, reward, done, info = self.env.step(act)
-        assert np.all(obs.target_dispatch == np.array([0.,  0., 20.,  0.,  0.]))
-        assert np.abs(np.sum(obs.actual_dispatch)) <= self.tol_one
-        th_disp = np.array([  0., -10.,  20.,   0., -10.])
-        assert self.compare_vect(obs.actual_dispatch, th_disp)
-        assert np.all(obs.prod_p <= self.env.gen_pmax)
-        assert np.all(obs.prod_p >= self.env.gen_pmin)
+        obs0, reward, done, info = self.env.step(act)
+        assert np.all(obs0.target_dispatch == np.array([0.,  0., 20.,  0.,  0.]))
+        assert np.abs(np.sum(obs0.actual_dispatch)) <= self.tol_one
+        th_disp = np.array([0., -10.,  20.,   0., -10.])
+        th_disp = np.array([0.,  -5.9259276,  20.,   0., -14.074072])
+        assert self.compare_vect(obs0.actual_dispatch, th_disp)
+        assert np.all(obs0.prod_p <= self.env.gen_pmax + self.tol_one)
+        assert np.all(obs0.prod_p >= self.env.gen_pmin - self.tol_one)
 
         act = self.env.action_space({"redispatch": [(2, 40.)]})
         obs, reward, done, info = self.env.step(act)
+        assert not info["is_dispatching_illegal"]
         assert np.all(obs.target_dispatch == np.array([0.,  0., 60.,  0.,  0.]))
-        th_disp = np.array([0., -23.49, 50.39, 0., -26.900002])
+        th_disp = np.array([0., -23.5,  50.4,   0., -26.900002])
         assert self.compare_vect(obs.actual_dispatch, th_disp)
-        assert np.all(obs.prod_p[:-1] <= self.env.gen_pmax[:-1])
-        assert np.all(obs.prod_p[:-1] >= self.env.gen_pmin[:-1])
+        assert np.all(obs.prod_p[:-1] <= self.env.gen_pmax[:-1] + self.tol_one)
+        assert np.all(obs.prod_p[:-1] >= self.env.gen_pmin[:-1] - self.tol_one)
+        assert np.all(obs.prod_p[:-1] - obs0.prod_p[:-1] >= - self.env.gen_max_ramp_down[:-1])
+        assert np.all(obs.prod_p[:-1] - obs0.prod_p[:-1] <= self.env.gen_max_ramp_up[:-1])
 
     def test_redispacth_non_dispatchable_generator(self):
         """ Dispatch a non redispatchable generator is ambiguous """
@@ -266,7 +276,7 @@ class BaseTestRedispatch(MakeBackend):
         # Check that generator 0 isn't redispatchable
         assert self.env.gen_redispatchable[0] == False
         # Check that generator 0 is off
-        assert self.env.gen_downtime[0] >= 1
+        assert self.env._gen_downtime[0] >= 1
 
         # Try to redispatch
         redispatch_act = self.env.action_space({"redispatch": [(0, 5.)]})
@@ -319,6 +329,7 @@ class BaseTestRedispatchChangeNothingEnvironment(MakeBackend):
                                names_chronics_to_backend=self.names_chronics_to_backend,
                                actionClass=BaseAction,
                                name="test_redisp_env2")
+        self.tol_one = self.env._tol_poly
 
     def tearDown(self):
         self.env.close()
@@ -335,7 +346,7 @@ class BaseTestRedispatchChangeNothingEnvironment(MakeBackend):
 
         # Check that generator 1 is off
         assert obs.prod_p[1] == 0
-        assert self.env.gen_downtime[1] >= 1
+        assert self.env._gen_downtime[1] >= 1
 
         # Try to redispatch generator 1
         redispatch_act = self.env.action_space({"redispatch": [(1, 5.)]})
@@ -357,6 +368,7 @@ class BaseTestRedispTooLowHigh(MakeBackend):
         self.env.gen_max_ramp_up[:] = 5000
         self.msg_ = 'Grid2OpException AmbiguousAction InvalidRedispatching NotEnoughGenerators "Attempt to use a ' \
                'redispatch action that does not sum to 0., but a'
+        self.tol_one = self.env._tol_poly
 
     def tearDown(self):
         self.env.close()
@@ -374,26 +386,26 @@ class BaseTestRedispTooLowHigh(MakeBackend):
         obs, reward, done, info = self.env.step(act)
         assert not done
         assert info["is_dispatching_illegal"] is False
-        assert np.all(self.env.target_dispatch == [-1., 0., 0., 0., 0.])
+        assert np.all(self.env._target_dispatch == [-1., 0., 0., 0., 0.])
         act = self.env.action_space({"redispatch": [0, 0]})
         obs, reward, done, info = self.env.step(act)
         assert not done
         assert info["is_dispatching_illegal"] is False
-        assert np.all(self.env.target_dispatch == [-1., 0., 0., 0., 0.])
+        assert np.all(self.env._target_dispatch == [-1., 0., 0., 0., 0.])
 
         # this one is not correct: too high decrease
         act = self.env.action_space({"redispatch": [0, self.env.gen_pmin[0] - self.env.gen_pmax[0]]})
         obs, reward, done, info = self.env.step(act)
         assert not done
         assert info["is_dispatching_illegal"]
-        assert np.all(self.env.target_dispatch == [-1., 0., 0., 0., 0.])
+        assert np.all(self.env._target_dispatch == [-1., 0., 0., 0., 0.])
 
         # this one is not correct: too high increase
         act = self.env.action_space({"redispatch": [0, self.env.gen_pmax[0] - self.env.gen_pmin[0] +2 ]})
         obs, reward, done, info = self.env.step(act)
         assert not done
         assert info["is_dispatching_illegal"]
-        assert np.all(self.env.target_dispatch == [-1., 0., 0., 0. ,0.])
+        assert np.all(self.env._target_dispatch == [-1., 0., 0., 0. ,0.])
 
     def test_error_message_notzerosum_oneshot(self):
         self.skipTest("Ok with new redispatching implementation")
@@ -426,6 +438,7 @@ class BaseTestDispatchRampingIllegalETC(MakeBackend):
         with warnings.catch_warnings():
             warnings.filterwarnings("ignore")
             self.env = make("rte_case14_test", test=True, backend=backend)
+        self.tol_one = self.env._tol_poly
 
     def tearDown(self):
         self.env.close()
@@ -456,10 +469,10 @@ class BaseTestDispatchRampingIllegalETC(MakeBackend):
         target_p_t = self.env.chronics_handler.real_data.data.prod_p[2, :]
         assert self.compare_vect(obsinit.prod_p[:-1], target_p_t[:-1])
         # only look at dispatchable generator, remove slack bus (last generator)
-        assert np.all(obs.prod_p[0:2] - obsinit.prod_p[0:2] <= obs.gen_max_ramp_up[0:2])
-        assert np.all(obs.prod_p[0:2] - obsinit.prod_p[0:2] >= -obs.gen_max_ramp_down[0:2])
-        assert np.all(obs.prod_p[0:2] >= obs.gen_pmin[0:2])
-        assert np.all(obs.prod_p[0:2] <= obs.gen_pmax[0:2])
+        assert np.all(obs.prod_p[0:2] - obsinit.prod_p[0:2] <= obs.gen_max_ramp_up[0:2] + self.tol_one)
+        assert np.all(obs.prod_p[0:2] - obsinit.prod_p[0:2] >= -obs.gen_max_ramp_down[0:2] - self.tol_one)
+        assert np.all(obs.prod_p[0:2] >= obs.gen_pmin[0:2] - self.tol_one)
+        assert np.all(obs.prod_p[0:2] <= obs.gen_pmax[0:2] + self.tol_one)
 
     def test_redispatch_noneedtocurtaildispact(self):
         self.skip_if_needed()
@@ -473,14 +486,14 @@ class BaseTestDispatchRampingIllegalETC(MakeBackend):
         act = self.env.action_space({"redispatch": [(0, +5)]})
         obs, reward, done, info = self.env.step(act)
         assert not done
-        assert np.all(self.env.target_dispatch == [5., 0., 0., 0., 0.])
+        assert np.all(self.env._target_dispatch == [5., 0., 0., 0., 0.])
         target_p = self.env.chronics_handler.real_data.data.prod_p[3, :]
         target_p_t = self.env.chronics_handler.real_data.data.prod_p[2, :]
         assert self.compare_vect(obsinit.prod_p[:-1], target_p_t[:-1])
         # only look at dispatchable generator, remove slack bus (last generator)
-        assert np.all(obs.prod_p[0:2] - obsinit.prod_p[0:2] <= obs.gen_max_ramp_up[0:2])
-        assert np.all(obs.prod_p[0:2] - obsinit.prod_p[0:2] >= -obs.gen_max_ramp_down[0:2])
-        assert np.all(np.abs(self.env.actual_dispatch - np.array([5., -2.5,  0.,  0., -2.5])) <= self.tol_one)
+        assert np.all(obs.prod_p[0:2] - obsinit.prod_p[0:2] <= obs.gen_max_ramp_up[0:2] + self.tol_one)
+        assert np.all(obs.prod_p[0:2] - obsinit.prod_p[0:2] >= -obs.gen_max_ramp_down[0:2] - self.tol_one)
+        assert np.all(np.abs(self.env._actual_dispatch - np.array([5., -2.5,  0.,  0., -2.5])) <= self.tol_one)
 
     def test_sum0_again(self):
         # perform a valid redispatching action
@@ -490,8 +503,8 @@ class BaseTestDispatchRampingIllegalETC(MakeBackend):
         act = self.env.action_space({"redispatch": [(0, +10)]})
         obs, reward, done, info = self.env.step(act)
         assert np.abs(np.sum(obs.actual_dispatch)) <= self.tol_one
-        indx_ok = self.env.target_dispatch != 0.
-        assert np.all(np.sign(self.env.actual_dispatch[indx_ok]) == np.sign(self.env.target_dispatch[indx_ok]))
+        indx_ok = self.env._target_dispatch != 0.
+        assert np.all(np.sign(self.env._actual_dispatch[indx_ok]) == np.sign(self.env._target_dispatch[indx_ok]))
 
     def test_sum0_again2(self):
         self.skip_if_needed()
@@ -503,15 +516,15 @@ class BaseTestDispatchRampingIllegalETC(MakeBackend):
         act = env.action_space({"redispatch": [(0, +5)]})
         obs, reward, done, info = env.step(act)
         assert np.abs(np.sum(obs.actual_dispatch)) <= self.tol_one
-        indx_ok = self.env.target_dispatch != 0.
-        assert np.all(np.sign(self.env.actual_dispatch[indx_ok]) == np.sign(self.env.target_dispatch[indx_ok]))
+        indx_ok = self.env._target_dispatch != 0.
+        assert np.all(np.sign(self.env._actual_dispatch[indx_ok]) == np.sign(self.env._target_dispatch[indx_ok]))
         donothing = env.action_space()
         obsinit, reward, done, info = env.step(donothing)
         act = env.action_space({"redispatch": [(0, -5)]})
         # act = env.action_space({"redispatch": [(0,0)]})
         obs, reward, done, info = env.step(act)
-        assert np.all(obs.prod_p[0:2] - obsinit.prod_p[0:2] <= obs.gen_max_ramp_up[0:2])
-        assert np.all(obs.prod_p[0:2] - obsinit.prod_p[0:2] >= -obs.gen_max_ramp_down[0:2])
+        assert np.all(obs.prod_p[0:2] - obsinit.prod_p[0:2] <= obs.gen_max_ramp_up[0:2] + self.tol_one)
+        assert np.all(obs.prod_p[0:2] - obsinit.prod_p[0:2] >= -obs.gen_max_ramp_down[0:2] - self.tol_one)
         assert np.abs(np.sum(obs.actual_dispatch)) <= self.tol_one
 
     def test_sum0_again3(self):
@@ -525,30 +538,30 @@ class BaseTestDispatchRampingIllegalETC(MakeBackend):
         act = env.action_space({"redispatch": [(0, +5)]})
         obs, reward, done, info = env.step(act)
         assert np.abs(np.sum(obs.actual_dispatch)) <= self.tol_one
-        indx_ok = self.env.target_dispatch != 0.
-        assert np.all(np.sign(self.env.actual_dispatch[indx_ok]) == np.sign(self.env.target_dispatch[indx_ok]))
-        assert np.all(obs.prod_p[0:2] - obs_init.prod_p[0:2] <= obs.gen_max_ramp_up[0:2])
-        assert np.all(obs.prod_p[0:2] - obs_init.prod_p[0:2] >= -obs.gen_max_ramp_down[0:2])
-        assert np.all(obs.actual_dispatch == np.array([5.0, -2.5, 0., 0., -2.5]))
+        indx_ok = self.env._target_dispatch != 0.
+        assert np.all(np.sign(self.env._actual_dispatch[indx_ok]) == np.sign(self.env._target_dispatch[indx_ok]))
+        assert np.all(obs.prod_p[0:2] - obs_init.prod_p[0:2] <= obs.gen_max_ramp_up[0:2] + self.tol_one)
+        assert np.all(obs.prod_p[0:2] - obs_init.prod_p[0:2] >= -obs.gen_max_ramp_down[0:2] - self.tol_one)
+        assert np.all(np.abs(obs.actual_dispatch - np.array([5.0, -2.5, 0., 0., -2.5])) <= self.tol_one)
         assert len(info['exception']) == 0
          # wait for the setpoint to be reached
         donothing = env.action_space()
         obsinit, reward, done, info = env.step(donothing)
-        assert np.all(obs.actual_dispatch == np.array([5.0, -2.5, 0., 0., -2.5]))
+        assert np.all(np.abs(obs.actual_dispatch - np.array([5.0, -2.5, 0., 0., -2.5])) <= self.tol_one)
         assert len(info['exception']) == 0
          # "cancel" action
         act = env.action_space({"redispatch": [(0, -5)]})
         obs, reward, done, info = env.step(act)
         assert not done
-        assert np.all(obs.prod_p[0:2] - obsinit.prod_p[0:2] <= obs.gen_max_ramp_up[0:2])
-        assert np.all(obs.prod_p[0:2] - obsinit.prod_p[0:2] >= -obs.gen_max_ramp_down[0:2])
+        assert np.all(obs.prod_p[0:2] - obsinit.prod_p[0:2] <= obs.gen_max_ramp_up[0:2] + self.tol_one)
+        assert np.all(obs.prod_p[0:2] - obsinit.prod_p[0:2] >= -obs.gen_max_ramp_down[0:2] - self.tol_one)
         assert np.abs(np.sum(obs.actual_dispatch)) <= self.tol_one
         assert len(info['exception']) == 0
          # wait for setpoint to be reached
         obsfinal, reward, done, info = env.step(donothing)
         assert not done
-        assert np.all(obsfinal.prod_p[0:2] - obs.prod_p[0:2] <= obs.gen_max_ramp_up[0:2])
-        assert np.all(obsfinal.prod_p[0:2] - obs.prod_p[0:2] >= -obs.gen_max_ramp_down[0:2])
+        assert np.all(obsfinal.prod_p[0:2] - obs.prod_p[0:2] <= obs.gen_max_ramp_up[0:2] + self.tol_one)
+        assert np.all(obsfinal.prod_p[0:2] - obs.prod_p[0:2] >= -obs.gen_max_ramp_down[0:2] - self.tol_one)
         assert np.abs(np.sum(obsfinal.actual_dispatch)) <= self.tol_one  # redispatching should sum at 0.
         assert np.sum(np.abs(obsfinal.actual_dispatch)) <= self.tol_one  # redispatching should be canceled by now
         assert len(info['exception']) == 0
@@ -572,12 +585,12 @@ class BaseTestDispatchRampingIllegalETC(MakeBackend):
             assert not done, "game over at iteration {}".format(i)
             assert len(info['exception']) == 0, "error at iteration {}".format(i)
             # NB: only gen 0 and 1 are included because gen 2,3 are renewables and gen 4 is slack bus
-            assert np.all(obs.prod_p[0:2] - obs_init.prod_p[0:2] <= obs.gen_max_ramp_up[0:2]), "above max_ramp for ts {}".format(i)
-            assert np.all(obs.prod_p[0:2] - obs_init.prod_p[0:2] >= -obs.gen_max_ramp_down[0:2]), "below min_ramp for ts {}".format(i)
+            assert np.all(obs.prod_p[0:2] - obs_init.prod_p[0:2] <= obs.gen_max_ramp_up[0:2] + self.tol_one), "above max_ramp for ts {}".format(i)
+            assert np.all(obs.prod_p[0:2] - obs_init.prod_p[0:2] >= -obs.gen_max_ramp_down[0:2] - self.tol_one), "below min_ramp for ts {}".format(i)
             try:
                 assert np.all(obs.prod_p[0:2] <= obs.gen_pmax[0:2]), "above pmax for ts {}".format(i)
             except:
-                pdb.set_trace()
+                pass
             assert np.all(obs.prod_p[0:2] >= -obs.gen_pmin[0:2]), "below pmin for ts {}".format(i)
             assert np.abs(np.sum(obs.actual_dispatch)) <= self.tol_one
 
@@ -587,10 +600,10 @@ class BaseTestDispatchRampingIllegalETC(MakeBackend):
                 break
 
         obs, reward, done, info = env.step(act)
-        assert np.all(obs.prod_p[0:2] - obs_init.prod_p[0:2] <= obs.gen_max_ramp_up[0:2]), "above max_ramp at the end"
-        assert np.all(obs.prod_p[0:2] - obs_init.prod_p[0:2] >= -obs.gen_max_ramp_down[0:2]), "above min_ramp at the end"
-        assert np.all(obs.prod_p[0:2] <= obs.gen_pmax[0:2]), "above pmax at the end"
-        assert np.all(obs.prod_p[0:2] >= -obs.gen_pmin[0:2]), "below pmin at the end"
+        assert np.all(obs.prod_p[0:2] - obs_init.prod_p[0:2] <= obs.gen_max_ramp_up[0:2] + self.tol_one), "above max_ramp at the end"
+        assert np.all(obs.prod_p[0:2] - obs_init.prod_p[0:2] >= -obs.gen_max_ramp_down[0:2] - self.tol_one), "above min_ramp at the end"
+        assert np.all(obs.prod_p[0:2] <= obs.gen_pmax[0:2] + self.tol_one), "above pmax at the end"
+        assert np.all(obs.prod_p[0:2] >= -obs.gen_pmin[0:2] - self.tol_one), "below pmin at the end"
         assert np.abs(np.sum(obs.actual_dispatch)) <= self.tol_one, "redisp not 0 at the end"
         # this redispatching is impossible because we ask to increase the value of the generator of 210
         # which is higher than pmax
@@ -604,6 +617,7 @@ class BaseTestLoadingAcceptAlmostZeroSumRedisp(MakeBackend):
         with warnings.catch_warnings():
             warnings.filterwarnings("ignore")
             self.env = make("rte_case14_test", test=True, backend=backend)
+        self.tol_one = self.env._tol_poly
 
     def tearDown(self):
         self.env.close()
