@@ -29,6 +29,9 @@ PROFILE_CODE = False
 if PROFILE_CODE:
     import cProfile
 
+import warnings
+warnings.simplefilter("error")
+
 
 class TestLoadingBackendPandaPower(unittest.TestCase):
     def setUp(self):
@@ -135,7 +138,7 @@ class TestLoadingBackendPandaPower(unittest.TestCase):
         assert self.compare_vect(injs_act, vect)
 
     def test_proper_voltage_modification(self):
-        do_nothing = self.env.helper_action_player({})
+        do_nothing = self.env.action_space({})
         obs, reward, done, info = self.env.step(do_nothing)  # should load the first time stamp
         vect = np.array([143.9, 139.1,   0.2,  13.3, 146. ])
         assert self.compare_vect(obs.prod_v, vect), "Production voltages setpoint have not changed at first time step"
@@ -145,7 +148,7 @@ class TestLoadingBackendPandaPower(unittest.TestCase):
 
     def test_number_of_timesteps(self):
         for i in range(287):
-            do_nothing = self.env.helper_action_player({})
+            do_nothing = self.env.action_space({})
             obs, reward, done, info = self.env.step(do_nothing)  # should load the first time stamp
         injs_act, *_ = self.env.backend.loads_info()
         vect = np.array([19.0, 87.9, 44.4, 7.2, 10.4, 27.5, 8.4, 3.2, 5.7, 12.2, 13.6])
@@ -156,7 +159,7 @@ class TestLoadingBackendPandaPower(unittest.TestCase):
         done = False
         i = 0
         while not done:
-            do_nothing = self.env.helper_action_player({})
+            do_nothing = self.env.action_space({})
             obs, reward, done, info = self.env.step(do_nothing)  # should load the first time stamp
             i += 1
         assert i == 287
@@ -177,7 +180,7 @@ class TestLoadingBackendPandaPower(unittest.TestCase):
             cp.enable()
         beg_ = time.time()
         cum_reward = dt_float(0.0)
-        do_nothing = self.env.helper_action_player({})
+        do_nothing = self.env.action_space({})
         while not done:
             obs, reward, done, info = self.env.step(do_nothing)  # should load the first time stamp
             cum_reward += reward
@@ -219,22 +222,22 @@ class TestIllegalAmbiguous(unittest.TestCase):
 
     def test_ambiguous_detected(self):
         self.skipTest("deprecated test as the reconnection is handled by backend action")
-        act = self.env.helper_action_player({"set_line_status": [(1, 1)]})
+        act = self.env.action_space({"set_line_status": [(1, 1)]})
         obs, reward, done, info = self.env.step(act)
         assert info['is_ambiguous']
         assert not info["is_illegal"]
 
     def test_notambiguous_correct(self):
-        act = self.env.helper_action_player({"set_line_status": [(1, -1)]})
+        act = self.env.action_space({"set_line_status": [(1, -1)]})
         obs, reward, done, info = self.env.step(act)
         assert not info['is_ambiguous']
         assert not info["is_illegal"]
         assert np.sum(obs.line_status) == 7
 
     def test_illegal_detected(self):
-        act = self.env.helper_action_player({"set_line_status": [(1, -1)]})
+        act = self.env.action_space({"set_line_status": [(1, -1)]})
         self.env.game_rules = RulesChecker(legalActClass=DefaultRules)
-        self.env.times_before_line_status_actionable[1] = 1
+        self.env._times_before_line_status_actionable[1] = 1
         obs, reward, done, info = self.env.step(act)
 
         # the action is illegal and it has not been implemented on the powergrid
@@ -374,6 +377,7 @@ class TestResetOk(unittest.TestCase):
         simobs, simr, simdone, siminfo = obs.simulate(self.env.action_space())
         assert np.all(simobs.topo_vect == 1)
 
+
 class TestAttachLayout(unittest.TestCase):
     def test_attach(self):
         my_layout = [(0, 0), (0, 400), (200, 400), (400, 400), (400, 0)]
@@ -387,16 +391,16 @@ class TestAttachLayout(unittest.TestCase):
                 dict_act = act.to_dict()
                 assert "grid_layout" in dict_act
                 assert dict_act["grid_layout"] == {k: [x,y] for k,(x,y) in zip(env.name_sub, my_layout)}
-                dict_ = env.helper_action_player.to_dict()
+                dict_ = env.action_space.to_dict()
                 assert "grid_layout" in dict_
                 assert dict_["grid_layout"] == {k: [x,y] for k,(x,y) in zip(env.name_sub, my_layout)}
-                dict_ = env.helper_action_env.to_dict()
+                dict_ = env._helper_action_env.to_dict()
                 assert "grid_layout" in dict_
                 assert dict_["grid_layout"] == {k: [x,y] for k,(x,y) in zip(env.name_sub, my_layout)}
-                dict_ = env.helper_observation.to_dict()
+                dict_ = env.observation_space.to_dict()
                 assert "grid_layout" in dict_
                 assert dict_["grid_layout"] == {k: [x,y] for k,(x,y) in zip(env.name_sub, my_layout)}
-                dict_ = env.opponent_action_space.to_dict()
+                dict_ = env._opponent_action_space.to_dict()
                 assert "grid_layout" in dict_
                 assert dict_["grid_layout"] == {k: [x,y] for k,(x,y) in zip(env.name_sub, my_layout)}
 
@@ -608,6 +612,105 @@ class TestDeactivateForecast(unittest.TestCase):
         # i check i can use it again
         sim_obs, *_ = obs.simulate(self.env1.action_space())
 
+    def _check_env_param(self, env, param):
+        assert env._ignore_min_up_down_times == param.IGNORE_MIN_UP_DOWN_TIME
+        assert env._forbid_dispatch_off == (not param.ALLOW_DISPATCH_GEN_SWITCH_OFF)
+
+        # type of power flow to play
+        # if True, then it will not disconnect lines above their thermal limits
+        assert env._no_overflow_disconnection == param.NO_OVERFLOW_DISCONNECTION
+        assert env._hard_overflow_threshold == param.HARD_OVERFLOW_THRESHOLD
+
+        # store actions "cooldown"
+        assert env._max_timestep_line_status_deactivated == param.NB_TIMESTEP_COOLDOWN_LINE
+        assert env._max_timestep_topology_deactivated == param.NB_TIMESTEP_COOLDOWN_SUB
+        assert env._nb_ts_reco == param.NB_TIMESTEP_RECONNECTION
+
+        assert np.all(env._nb_timestep_overflow_allowed == param.NB_TIMESTEP_OVERFLOW_ALLOWED)
+
+        # hard overflow part
+        assert env._env_dc == param.ENV_DC
+        assert np.all(env._nb_timestep_overflow_allowed == param.NB_TIMESTEP_OVERFLOW_ALLOWED)
+        assert env._max_timestep_line_status_deactivated == param.NB_TIMESTEP_COOLDOWN_LINE
+
+    def test_change_parameters_basic(self):
+        """
+        this is basic test of the env.change_parameters() function
+
+        It only checks the right parameters are used for the environment but it do not currently
+        check the observation (with the cooldown for example)
+        """
+        param = Parameters()
+        param.NO_OVERFLOW_DISCONNECTION = True
+        param.MAX_SUB_CHANGED = 5
+        param.MAX_LINE_STATUS_CHANGED = 5
+        param.NB_TIMESTEP_COOLDOWN_LINE = 5
+        param.NB_TIMESTEP_COOLDOWN_SUB = 5
+        real_orig_param = copy.deepcopy(self.env1.parameters)
+        self.env1.change_parameters(param)
+        # parameters have not changed yet
+        _ = self.env1.step(self.env1.action_space())
+        self._check_env_param(self.env1, real_orig_param)
+
+        # reset triggers the change of parameters
+        self.env1.reset()
+        self._check_env_param(self.env1, param)
+
+    def test_change_parameters_forecast(self):
+        """
+        this is basic test of the env.change_forecast_parameters() function
+
+        It only checks the right parameters are used for the environment (or obs_env) but it do not currently
+        check the observation (with the cooldown for example)
+        """
+        param = Parameters()
+        param.NO_OVERFLOW_DISCONNECTION = True
+        param.MAX_SUB_CHANGED = 5
+        param.MAX_LINE_STATUS_CHANGED = 5
+        param.NB_TIMESTEP_COOLDOWN_LINE = 5
+        param.NB_TIMESTEP_COOLDOWN_SUB = 5
+        param.ENV_DC = True
+        real_orig_param = copy.deepcopy(self.env1.parameters)
+        self.env1.change_forecast_parameters(param)
+        # in these first checks, parameters are not modified
+        self._check_env_param(self.env1, real_orig_param)
+        self._check_env_param(self.env1._helper_observation.obs_env, real_orig_param)
+
+        obs, *_ = self.env1.step(self.env1.action_space())
+        _ = obs.simulate(self.env1.action_space())
+        self._check_env_param(self.env1, real_orig_param)
+        self._check_env_param(self.env1._helper_observation.obs_env, real_orig_param)
+
+        # reset triggers the modification
+        obs = self.env1.reset()
+        _ = obs.simulate(self.env1.action_space())
+        self._check_env_param(self.env1, real_orig_param)
+        self._check_env_param(self.env1._helper_observation.obs_env, param)
+
+    def test_change_parameters_forecast_fromissue_128(self):
+        """
+        this is basic test of the env.change_forecast_parameters() function
+
+        It only checks the right parameters are used for the environment (or obs_env) but it do not currently
+        check the observation (with the cooldown for example)
+
+        This is the example taken from https://github.com/rte-france/Grid2Op/issues/128 (first remak)
+        """
+
+        # modify the parmeters for simulate
+        param = Parameters()
+        param.MAX_SUB_CHANGED = 9999
+        param.MAX_LINE_STATUS_CHANGED = 9999
+        self.env1.change_forecast_parameters(param)
+        self.env1.reset()
+
+        obs, *_ = self.env1.step(self.env1.action_space())
+        # and you can simulate the impact of action modifying as many substation as you want, for example
+        act = self.env1.action_space({"set_bus": {"lines_or_id": [(0, 2), (10, 2)]}})
+        # this action modfies both substation 0 and substation 8
+        sim_o, sim_r, sim_d, sim_info = obs.simulate(act)
+        assert sim_info["is_illegal"] is False
+
 
 if __name__ == "__main__":
     unittest.main()
